@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: compute.tcl
-#  Last change:  21 October 2004
+#  Last change:  4 May 2006
 #
 # Includes contributions by
 #  - Matt Martin (matt.martin@ieee.org) marked "MGM contribution"
@@ -142,6 +142,7 @@ proc FormatLatLong {latd longd pfmt} {
     #  degrees in latitude/longitude position format
     # result is the position representation: a list whose first two elements
     #  are lat and long in degrees, followed by lat and long in external format
+    global COUTFMT
 
     if { $latd < 0 } {
 	set lax [expr -$latd] ; set hlat S
@@ -150,8 +151,8 @@ proc FormatLatLong {latd longd pfmt} {
 	set lox [expr -$longd] ; set hlng W
     } else { set lox $longd ; set hlng E }
     if { $pfmt == "GRA" } {
-	set lax [format %.5f [expr $latd/0.9]]
-	set lox [format %.5f [expr $longd/0.9]]
+	set lax [format $COUTFMT(GRA) [expr $latd/0.9]]
+	set lox [format $COUTFMT(GRA) [expr $longd/0.9]]
 	return [list $latd $longd $lax $lox]
     }
     return [list $latd $longd "$hlat[ExtDegrees $pfmt $lax]" \
@@ -163,6 +164,7 @@ proc ExtDegrees {pformt degs} {
     # $pformt in {DMS, DMM, DDD, DMSsimpl}
     #  DMSsimpl is similar to DMS, but for values less than 1 degree the
     #  representation is either MM'SS.S" or SS.S"
+    global COUTFMT
 
     switch -glob $pformt {
 	DMS* {
@@ -176,13 +178,14 @@ proc ExtDegrees {pformt degs} {
 	      set s [expr ($degs-$m)*60]
 	      if { $s > 59.95 } { set s 0 ; incr m }
 	      if { $m > 59 } { set m 0 ; incr d }
+	      set sf $COUTFMT(sec)
 	      if { $d > 0 || $pformt == "DMS" } {
-		  return [format "%d %02d %04.1f" $d $m $s]
+		  return [format "%d %02d $sf" $d $m $s]
 	      }
 	      if { $m > 0 } {
-		  return [format "%02d'%04.1f" $m $s]
+		  return [format "%02d'$sf" $m $s]
 	      }
-	      return [format "%04.1f\"" $s]
+	      return [format "$sf\"" $s]
 	}
 	DMM {
 	      if { $degs < 0 } {
@@ -192,10 +195,10 @@ proc ExtDegrees {pformt degs} {
 	      set degs [expr ($degs-$d)*60]
 	      set d [expr $sign*$d]
 	      if { $degs > 59.995 } { set degs 0 ; incr d }
-	      return [format "%d %06.3f" $d $degs]
+	      return [format "%d $COUTFMT(min)" $d $degs]
 	}
 	DDD {
-	      return [format "%.5f" $degs]
+	      return [format $COUTFMT(deg) $degs]
 	}
     }
 }
@@ -215,7 +218,7 @@ proc ConvertPos {pfrmt zone x y datum npfrmt} {
     # values of $x, $y are checked, as well as suitability of $datum
     #  and of validity of new format
     # return -1 on error
-    global POSTYPE MESS TXT
+    global POSTYPE MESS TXT COUTFMT
 
     switch [set ptype $POSTYPE($pfrmt)] {
 	latlong {
@@ -238,8 +241,8 @@ proc ConvertPos {pfrmt zone x y datum npfrmt} {
 		return -1
 	    }
 	    if { $npfrmt == "DDD" } {
-		return [list $y $x "${hlat}[format "%.5f" $la]" \
-			           "${hlng}[format "%.5f" $lo]"]
+		return [list $y $x "${hlat}[format $COUTFMT(deg) $la]" \
+			           "${hlng}[format $COUTFMT(deg) $lo]"]
 	    }
 	    set la $y ; set lo $x
 	}
@@ -260,8 +263,8 @@ proc ConvertPos {pfrmt zone x y datum npfrmt} {
 	    }
 	    if { $npfrmt == $pfrmt } {
 		if { $ptype == "grid" } {
-		    return [list $la $lo $zone $x $y]
-		} else { return [list $la $lo $x $y] }
+		    return [linsert $p end $zone $x $y]
+		} else { return [linsert $p end $x $y] }
 	    }
 	}
 	default {
@@ -980,7 +983,7 @@ proc DateToSecsFrom {y m d h mn s year0} {
 
 proc DateFromSecs {secs} {
     # build date from seconds ellapsed since beginning of $YEAR0, not
-    #  necessarily(a leap year, using default format
+    #  necessarily a leap year, using default format
     global DateFormat
 
     return [eval FormatDate $DateFormat [DateIntsFromSecs $secs]]
