@@ -137,6 +137,10 @@ proc Hgraph {k what args} {
 	# a time axis is always used in this case
 	set xmin 0
 	foreach "k valids secs0 xmax ymin ymax" [ClimbRateData $k] {}
+	if { $valids == "" } {
+	    GMMessage $MESS(missingdata)
+	    return true
+	}
     } else {
 	# initializations
 	set xmin 99999
@@ -530,6 +534,7 @@ proc ClimbRateData {salts} {
     #  is applied to the altitude values if there is enough data
     #  (at least 10 altitude values, so that 2 filtered values can be
     #  computed)
+    # return empty list if not enough data
     # return list with: list of pairs with seconds from first valid rate
     #  and climb rate, length of this list, seconds for the first valid
     #  rate, maximum value of seconds from first valid rate, minimum and
@@ -564,6 +569,10 @@ proc ClimbRateData {salts} {
 	    }
 	}
     }
+    if { $secs0 == "" } {
+	# no time data
+	return ""
+    }	
     set secs0 [expr round($secs0)]
     # is there enough data to filter?
     set scrs ""
