@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: garmin.tcl
-#  Last change:  22 November 2004
+#  Last change:  6 April 2006
 #
 # Includes contributions by
 #  - David Gardner (djgardner@users.sourceforge.net) marked "DJG contribution"
@@ -30,8 +30,12 @@
  #  "Garmin GPS Interface Specification", April 13 1999, 001-00063-00 Rev. 2
  #  "Garmin GPS Interface Specification", December 6 1999, 001-00063-00 Rev. 3
  #  "Garmin GPS Interface Specification", April 24 2004, 001-00063-00 Rev. A
+ #  "Garmin GPS Interface Specification", September 16 2004, 001-000063-00
+ #   Rev. B
 
-
+ # Garmin USB protocol is implemented assuming that the USB port is driven by
+ #  the garmin_gps Linux kernel driver by Hermann Kneissel and part of the
+ #  official 2.6 Linux kernel releases
 
 # logging the communication: see serial.tcl
 
@@ -44,6 +48,8 @@ set TestRepeated 0
 ### state of the communication
 set Request idle
 set Jobs ""
+
+ # serial
 set SInState idle
 set SInPacketState start
 set SOutBusy 0
@@ -53,12 +59,137 @@ set PkInState idle
 set PkLastData ""
 set FirstACKNAK 1
 
+ # general
 set NoInProc 1
 set GetPVT 0
 set STextPVT 0
 
+##### opening USB port
+
+proc OpenUSBFailed {} {
+    # open USB port to use the Garmin USB protocol, and log file if needs be
+    #  initialise USB driver
+    # return 1 on failure
+    global SERIALPORT USBFILE GUSB USBPackets USBMissing USBBuff Eof MESS
+
+    if { [catch {set USBFILE [open $SERIALPORT r+]}] } {
+	GMMessage [format $MESS(badserial) $SERIALPORT]
+	return 1
+    }
+    fconfigure $USBFILE -blocking 0 -translation binary
+    # initialise USB driver
+    #  change to USB mode
+    puts -nonewline $USBFILE \
+	[DataToStr "long long long long" \
+	     [list $GUSB(LAYERID,PRIVATE) $GUSB(PRIV,PID,SET,MODE) \
+		  4 $GUSB(MODE,NATIVE)]]
+    flush $USBFILE
+    #  driver can be set to debug mode with the packet
+    #    $GUSB(LAYERID,PRIVATE) $GUSB(PRIV,PID,SET,DEBUG) 4 0xffff
+
+    # send start session request to receiver
+    puts -nonewline $USBFILE \
+	[DataToStr "long long long" \
+	     [list $GUSB(LAYERID,TRANSPORT) \
+		  $GUSB(PID,TRANSPORT,START,SESSION,REQ) 0]]
+    # read and discard anything there is to be read
+    # should have waited for a session started packet at the transport level
+    after 2
+    read $USBFILE
+    # initialise input and open log file
+    set Eof 0
+    set USBPackets "" ; set USBMissing 0 ; set USBBuff ""
+    fileevent $USBFILE readable ReadUSB
+    OpenSerialLog
+    return 0
+}
+
+##### closing the serial or USB connection
+
+proc CloseConnection {} {
+    # close the connection after a turn off command or a change from
+    #  the serial or USB Garmin protocols
+    # receiver types and info are restored to the original lists
+    #  as a different receiver may be connected next
+    global GPSState Eof GPSProtocol USBFILE GUSB SRLFILE NoGarmin \
+	RECTYPES RECINFO ORIGRECTYPES ORIGRECINFO
+
+    Log "CC> closing down..."
+    set GPSState offline
+    set Eof 1
+    ResetSerial
+    if { $GPSProtocol == "garmin_usb" } {
+	# change USB driver to serial mode
+	Log "CC> changing to serial mode..."
+	puts -nonewline $USBFILE \
+	    [DataToStr "long long long long" \
+		 [list $GUSB(LAYERID,PRIVATE) $GUSB(PRIV,PID,SET,MODE) \
+		      4 $GUSB(MODE,GARMIN,SERIAL)]]
+	flush $USBFILE
+	close $USBFILE
+    } else {
+	close $SRLFILE
+    }
+    DisableGPS
+    set NoGarmin 1
+    set RECTYPES $ORIGRECTYPES ; set RECINFO $ORIGRECINFO
+    Log "CC> closed\n"
+    return
+}
+
 ##### low-level read procedures
 
+# USB protocol
+
+proc ReadUSB {} {
+    # read bytes and form a complete Garmin USB packet
+    # complete packets are kept in the list $USBPackets and
+    #  proc UseUSBInput called to process them when needed
+    global USBFILE Eof USBPackets NoInProc GUSB USBMissing USBBuff USBPId RPID
+
+    if { $Eof } { return }
+    if { $USBMissing == 0 } {
+	# read USB packet header
+	set buff [read $USBFILE 12]
+	if { [set n [string length $buff]] != 12 } {
+	    Log "RU> bad packet header, only $n bytes not 12; discarded"
+	    return
+	}
+	foreach "layerid USBPId USBMissing" \
+	    [UnPackData [split $buff ""] "long long long"] { break }
+	Log "RU> layerid=$layerid, pid=$USBPId, length=$n"
+	if { $layerid != $GUSB(LAYERID,APPL) || \
+		 [catch {set USBPId $RPID($USBPId)}] } {
+	    Log "RU> bad layer id or pid; discarding all data to be read"
+	    read $USBFILE
+	    set USBMissing 0
+	    return
+	}
+	set USBBuff ""
+    }
+    # read USB data
+    set pdata [read $USBFILE $USBMissing]
+    append USBBuff $pdata
+    if { [set nn [string length $pdata]] != $USBMissing } {
+	Log "RU> packet data, only $nn bytes not $USBMissing"
+	set USBMissing [expr $USBMissing-$nn]
+	return
+    }
+    Log "RU> complete packet got: length $nn"
+    lappend USBPackets [list $USBPId $USBBuff]
+    set USBMissing 0
+    if { $NoInProc } { after 0 UseUSBInput }
+
+    if { [set Eof [eof $USBFILE]] } {
+	Log "RU> at eof"
+	set GPSState offline
+	close $USBFILE
+    }
+    return
+}
+
+# line-oriented protocols
+
 proc ProcLineChar {char} {
     # process char received when expecting a line of input
     # call $ProcProcLine to process line (as string) when getting a line-feed
@@ -85,29 +216,81 @@ proc ProcLineChar {char} {
 
 ##### link layer
 
-proc SendPacket {pid tvals} {
-    # initialize a send packet action
+# USB protocol
+
+proc UseUSBInput {} {
+    # use packets read from USB channel in list $USBPackets
+    global USBPackets NoInProc
+
+    set NoInProc 0
+    while 1 {
+	set queue $USBPackets ; set USBPackets ""
+	foreach pak $queue {
+	    foreach "pid pdata" $pak {}
+	    set pdata [split $pdata ""]
+	    LogBytes "UUI> got packet pid=$pid; contents:" $pdata
+	    ProcInPacket $pid $pdata
+	}
+	update
+	if { $USBPackets == "" } { break }
+    }
+    set NoInProc 1
+    return
+}
+
+proc SendUSBPacket {pid types vals} {
+    # send a Garmin USB packet
+    #  $pid is the packet identifier name
+    #  $types is list of types aligned with
+    #  $vals a list of values
+    # assume that the packet is actually sent and call proc SentPacket
+    #  unless the size exceeds 64K-1
+    global USBFILE GUSB PID
+
+    if { [catch {set pdata [DataToStr $types $vals]}] } {
+	Log "SUP> BUG, bad data: pid=$pid\n\ttypes=$types\nvals=$vals"
+	return
+    }
+    if { [set size [string length $pdata]] > 65535 } {
+	Log "SUP> too much data ($size>65535)"
+	return
+    }
+    set pak [DataToStr "long long long" \
+		 [list $GUSB(LAYERID,APPL) $PID($pid) $size]]
+    append pak $pdata
+    LogBytes "SUP> sending packet $pid with:" [split $pak ""]
+    puts -nonewline $USBFILE $pak
+    flush $USBFILE
+    SentPacket $pid
+    return
+}
+
+# serial protocol
+
+proc SendSPacket {pid types vals} {
+    # initialize a send packet action to serial channel
     # wait for termination of a previous similar operation
     #  $pid is the packet identifier name
-    #  $tvals is a pair with a list of types and a list of values
+    #  $types is list of types aligned with
+    #  $vals a list of values
     # uses the following global variable that may be reset elsewhere:
     #  $SOutBusy a flag set if a packet is being sent; it is
-    #             cleared by DoSendPacket (for ACK/NAK packets)
+    #             cleared by DoSendSPacket (for ACK/NAK packets)
     #             or by ProcACKNAK (after a valid ACK is received)
     global SInState SOutBusy PID ACK NAK \
 	    SOutPID SOutBData SOutDSum SOutBPID SOutSize
 
     if { $SOutBusy } {
-	after 50 "SendPacket $pid {$tvals}"
+	after 50 "SendSPacket $pid {$types} {$vals}"
 	return
     }
     if { [catch {set SOutBPID $PID($pid)}] } {
-	Log "SP> bad PID: $pid"
+	Log "SSP> bad PID: $pid"
 	return
     }
-    set SOutBData [PackData $tvals]
+    set SOutBData [PackData $types $vals]
     if { [set SOutSize [llength $SOutBData]] > 255 } {
-	Log "SP> too much data ($SOutSize>255)"
+	Log "SSP> too much data ($SOutSize>255)"
 	return
     }
     Log "SP> sending packet $pid with:"
@@ -115,14 +298,14 @@ proc SendPacket {pid tvals} {
     foreach ch $SOutBData {
 	binary scan $ch "c" v
 	incr SOutDSum $v
-	Log "SP>   [expr ($v+256)%256]"
+	Log "SSP>   [expr ($v+256)%256]"
     }
-    DoSendPacket
+    DoSendSPacket
     return
 }
 
-proc DoSendPacket {} {
-    # do the actual sending of the packet initiated by SendPacket;
+proc DoSendSPacket {} {
+    # do the actual sending of the packet initiated by SendSPacket;
     # wait for termination of get operations in progress unless the
     #  packet is an ACK/NAK one
     # this proc may be called for re-sending the packet if a NAK is
@@ -131,17 +314,17 @@ proc DoSendPacket {} {
 	    SOutPID SOutBData SOutDSum SOutBPID SOutSize SRLFILE
 
     if { $SOutBPID!=$ACK && $SOutBPID!=$NAK && $SInState == "get" } {
-	after 50 DoSendPacket
+	after 50 DoSendSPacket
         return
     }
-    Log "DSP> sending packet $SOutPID"
+    Log "DSSP> sending packet $SOutPID"
     if { $SOutBPID!=$ACK && $SOutBPID!=$NAK } {
 	set SInPacketState start ; set SInState put
     }
     set datasum [expr (-$SOutDSum-$SOutBPID-$SOutSize)&255]
     if { [catch {puts -nonewline $SRLFILE \
 	              [binary format ccc $DLE $SOutBPID $SOutSize]}] } {
-        Log "DSP> error writing to $SRLFILE"
+        Log "DSSP> error writing to $SRLFILE"
         set SOutBusy 0
         if { $SOutBPID!=$ACK && $SOutBPID!=$NAK } { set SInState idle }
 	return
@@ -160,7 +343,7 @@ proc DoSendPacket {} {
     }
     puts -nonewline $SRLFILE [binary format "ccc" $datasum $DLE $ETX]
     flush $SRLFILE
-    Log "DSP> size=$SOutSize, checksum=$datasum"
+    Log "DSSP> size=$SOutSize, checksum=$datasum"
     if { $SOutBPID==$ACK || $SOutBPID==$NAK } { set SOutBusy 0 }
     return
 }
@@ -171,7 +354,7 @@ proc SendACKNAK {what pid} {
     #  $pid is packet id name to not-/acknowledge
     global PID PDTYPE
 
-    SendPacket $what [list $PDTYPE($what) $PID($pid)]
+    SendSPacket $what $PDTYPE($what) $PID($pid)
     return
 }
 
@@ -216,7 +399,7 @@ proc ProcACKNAK {pid data} {
 		incr SRecACKs
 		if { $bpid != $SOutBPID } {
 		    Log "PAN> ACK for wrong packet $bpid, expected $SOutPID"
-		    DoSendPacket
+		    DoSendSPacket
 		} else {
 		    SentPacket $SOutPID
 		    set SOutBusy 0
@@ -236,7 +419,7 @@ proc ProcACKNAK {pid data} {
 		if { $bpid != $SOutBPID } {
 		    Log "PAN> NAK for wrong packet $bpid, expected $SOutPID"
 		}
-		DoSendPacket
+		DoSendSPacket
 	    } else {
 		Log "PAN> NAK for $bpid when output is idle"
 	    }
@@ -245,7 +428,7 @@ proc ProcACKNAK {pid data} {
 	    # $SOutBusy must be 1
 	    GMMessage $MESS(noACKNAK)
 	    # Log "PAN> $pid when sending $SOutPID; resending"
-	    # DoSendPacket
+	    # DoSendSPacket
 	    Log "PAN> $pid when sending $SOutPID; resetting"
 	    ResetSerial
 	}
@@ -316,16 +499,8 @@ proc ProcChar {char} {
 			if { [PacketOK] } {
 			    set SInPacketState block
 			    if { $NotACKNAK } {
-				if { $PacketID == "PrdData" } {
-				    # save product data
-				    set PrdDATA $PacketData
-				    # wait for the Protocol Capability packet
-				    set PkInState A001
-				    SendACKNAK ACK PrdData
-				} else {
-				    SendACKNAK ACK $PacketID
-				    ProcInPacket $PacketID "$PacketData"
-				}
+				SendACKNAK ACK $PacketID
+				ProcInPacket $PacketID "$PacketData"
 		            } else {
 				ProcACKNAK $PacketID "$PacketData"
 			    }
@@ -389,9 +564,6 @@ proc ProcInPacket {pid data} {
     #                packets) or to name of protocol the incoming packet is
     #                is expected to conform to
     #  $GetPVT      is set if PVT data packets are expected
-    # this proc is not used anymore for A000 whose data is dealt with
-    #  when A001 terminates (see below) or fails to show up (see proc
-    #  AbortComm)
     global PkInState PkInPrevState PkInData PkInRTsData PkInCount CurrPSPID \
 	    PDTYPE SPckts GetPVT PkLastPID PkLastData Jobs \
 	    TestRepeated PrdDATA
@@ -421,14 +593,30 @@ proc ProcInPacket {pid data} {
 	    Log "PP> asking for non-available protocol"
 	    set PkInState idle
 	}
+	A000 {
+	    # product data protocol
+	    if { [GoodPID $pid PrdData] } {
+		# save product data
+		set PrdDATA $data
+		# wait for the Protocol Capability packet
+		set PkInState A001
+		# the data will be dealt with either when A001 terminates
+		# (see below) or fails to show up (see proc AbortComm)
+		return
+	    }
+	}
 	A001 {
 	    # protocol capability protocol
+	    if { $pid == "PrdExtData" } {
+		# this data is to be discarded; keep
+		#  waiting for the protocol capability packet
+		return
+	    }
 	    if { [GoodPID $pid PArray] } {
-		# the Product Data packet must be processed now
+		# the Product Data information must be processed now
 		# kill timeout alarm
 		after cancel [lindex $Jobs 0]
 		set PkInState idle
-		global PrdDATA
 		EndInProt A000 [UnPackData $PrdDATA $PDTYPE(PrdData)]
 		EndInProt A001 [UnPackData $data $PDTYPE(PArray)]
 	    }
@@ -584,7 +772,7 @@ proc ProcInPacket {pid data} {
 	    # expecting end of transfer on A500 protocol
 	    EndOfTransfer $pid $data AL A500 $PkInData
 	}
-	D500 -  D501 {
+	D500 -  D501 -  D550 -  D551 {
 	    # AL data; product specific
 	    if { [GoodPID $pid ALData] } {
 		lappend PkInData "[UnPackData $data $PDTYPE($PkInState)]"
@@ -625,6 +813,7 @@ proc ProcInPacket {pid data} {
 	    }
 	}
 	A011 -
+	A101 -
 	A400 -  A600 -  A650 -  A700 -
 	default {
 	    Log "$PkInState not supported"
@@ -690,19 +879,21 @@ proc GoodCMD {cmd wanted} {
 
 proc SendData {type args} {
     # start transfer to receiver
-    # first packet sent will hopefully be ACK-ed, what will fire up
-    #  (in sequence) ProcChar, ProcACKNAK, SentPacket
+    # first packet sent will, in serial mode, hopefully be ACK-ed, what
+    #  will fire up (in sequence) ProcChar, ProcACKNAK, SentPacket; in
+    #  USB mode SentPacket will be called after the packet is sent
     #   $type in {product, abort, turnOff, get, put, start, stop}
     #   $args void unless
-    #       $type==get: in {WP RT TR PosnData DtTmData}; not yetin {AL Prx}
-    #       $type==put: 1st arg in {WP RT TR PosnData DtTmData}; not yet
+    #       $type==get: in {WP RT TR PosnData DtTmData}; not yet
+    #                    in {AL Prx}
+    #       $type==put and 1st arg in {WP RT TR PosnData DtTmData}; not yet
     #                    in {AL Prx}
     #                   2nd arg is list of indices
     #       $type in {start, stop}: in {PVT}
     global PDTYPE PRTHNID PkInState SOutBusy CurrPSPID CMD Command CommArgs \
 	    PkOutState PkOutData PkOutCount PkOutStart PkOutWhat PkTRSegs \
-	    PkOutSaved RTIdNumber RTWPoints RTStages TRTPoints TRSegStarts \
-	    PrdDATA MESS PTRHNID
+	    PkOutSaved PkOutInfo RTIdNumber RTWPoints RTStages TRTPoints \
+	    TRSegStarts PrdDATA MESS PTRHNID ProcSendPacket
     # DJG contribution
     global RTNextNumber AutoNumRts RTSendNumber
     #--
@@ -711,29 +902,32 @@ proc SendData {type args} {
     switch $type {
 	product {
 	    set PrdDATA ""
-	    SendPacket PrdReq [list $PDTYPE(PrdReq) 0]
+	    $ProcSendPacket PrdReq $PDTYPE(PrdReq) 0
 	}
 	abort {
-	    SendPacket CmdData [list int $CMD(Abort)]
+	    $ProcSendPacket CmdData int [list $CMD(Abort)]
 	}
 	turnOff {
-	    SendPacket CmdData [list int $CMD(TurnOff)]
-	    # the receiver may not send an ACK before turning off
+	    $ProcSendPacket CmdData int $CMD(TurnOff)
+	    # do not wait for an answer
+	    CloseConnection
 	}
 	start {
 	    # $args in {PVT}
-	    SendPacket CmdData [list int $CMD(Start$args)]
+	    $ProcSendPacket CmdData int $CMD(Start$args)
 	}
 	stop {
 	    # $args in {PVT}
-	    SendPacket CmdData [list int $CMD(Stop$args)]
+	    $ProcSendPacket CmdData int $CMD(Stop$args)
 	}
 	get {
-	    # $args in {WP RT TR PosnData DtTmData}; not implemented: {AL Prx}
-	    SendPacket CmdData [list int $CMD(Xfr$args)]
+	    # $args in {WP RT TR PosnData DtTmData}
+	    #  not implemented: {AL Prx}
+	    $ProcSendPacket CmdData int $CMD(Xfr$args)
 	}
 	put {
 	    set putwhat [lindex $args 0]
+	    set ixs [lindex $args 1]
 	    if { $putwhat == "Prx" || $putwhat == "AL" } {
 		Log "put $putwhat not implemented"
 		return
@@ -742,13 +936,13 @@ proc SendData {type args} {
 	    set p $CurrPSPID($putwhat)
 	    switch $putwhat {
 		WP {
-		    set ixs [lindex $args 1] ; set n [llength $ixs]
 		    set PkOutStart XfrWP
 		    switch $p {
 			A100 {
-			    set PkOutData $ixs ; set PkOutCount $n
+			    set PkOutData $ixs
+			    set PkOutCount [llength $ixs]
 			    set PkOutState $CurrPSPID(WPData)
-			    SendPacket Records [list int $n]
+			    $ProcSendPacket Records int $PkOutCount
 			}
 			default {
 			    Log "protocol $p not implemented"
@@ -758,7 +952,6 @@ proc SendData {type args} {
 		}
 		RT {
 		    set nolk [string compare $CurrPSPID(RT) A201]
-		    set ixs [lindex $args 1]
 		    set n 0 ; set badrts 0
 		    set data ""
 		    # DJG contribution: do autonumbering of RTs if the device 
@@ -824,7 +1017,7 @@ proc SendData {type args} {
 			A201 {
 			    set PkOutData $data ; set PkOutCount $n
 			    set PkOutState $CurrPSPID(RTHeader)
-			    SendPacket Records [list int $n]
+			    $ProcSendPacket Records int $PkOutCount
 			}
 			default {
 			    Log "protocol $p not implemented"
@@ -833,7 +1026,6 @@ proc SendData {type args} {
 		    }
 		}
 		TR {
-		    set ixs [lindex $args 1]
 		    set n 0 ; set badtrs 0
 		    set PkTRSegs 0
 		    foreach tr $ixs {
@@ -870,7 +1062,7 @@ proc SendData {type args} {
 			}
 
 		    }
-		    SendPacket Records [list int $n]
+		    $ProcSendPacket Records int $n
 		}
 		PosnData -
 		DtTmData {
@@ -887,7 +1079,8 @@ proc SentPacket {pid} {
     # deal with continuation after a packet has been successfully sent
     global Command CommArgs PkInState CurrPSPID SInState \
 	     PkOutState PkOutWhat PkOutCount PkOutData PkOutStart PkOutSaved \
-	     CMD RTWPoints RTStages Jobs Request SPckts RPID
+	     CMD RTWPoints RTStages Jobs Request SPckts RPID \
+	     ProcSendPacket
 
     incr SPckts
     Log "StP> packet for $pid sent"
@@ -923,33 +1116,35 @@ proc SentPacket {pid} {
 	}
 	Records {
 	    if { $PkOutCount == 0 } {
-		SendPacket XfrCmpl [list int $CMD($PkOutStart)]
+		$ProcSendPacket XfrCmpl int $CMD($PkOutStart)
 	    } else {
 		incr PkOutCount -1
 		switch $PkOutWhat {
 		    WP {
-			SendPacket WPData [PrepData]
+			foreach "ts vs" [PrepData] {}
+			$ProcSendPacket WPData $ts $vs
 		    }
 		    RT {
-			SendPacket RTHeader [PrepData]
+			foreach "ts vs" [PrepData] {}
+			$ProcSendPacket RTHeader $ts $vs
 		    }
 		    TR {
 			set PkOutSaved ""
 			set pid $PkOutState
 			set PkOutState $CurrPSPID($pid)
-			SendPacket $pid [PrepData]
-		    }
-		    default {
+			foreach "ts vs" [PrepData] {}
+			$ProcSendPacket $pid $ts $vs
 		    }
 		}
 	    }
 	}
 	WPData {
 	    if { $PkOutCount == 0 } {
-		SendPacket XfrCmpl [list int $CMD($PkOutStart)]
+		$ProcSendPacket XfrCmpl int $CMD($PkOutStart)
 	    } else {
 		incr PkOutCount -1
-		SendPacket $pid [PrepData]
+		foreach "ts vs" [PrepData] {}
+		$ProcSendPacket $pid $ts $vs
 	    }
 	}
 	RTHeader {
@@ -976,9 +1171,10 @@ proc SentPacket {pid} {
 	    }
 	    if { [set PkOutCount $n] } {
 		incr PkOutCount -1
-		SendPacket RTWPData [PrepData]
+		foreach "ts vs" [PrepData] {}
+		$ProcSendPacket RTWPData $ts $vs
 	    } else {
-		SendPacket XfrCmpl [list int $CMD($PkOutStart)]
+		$ProcSendPacket XfrCmpl int $CMD($PkOutStart)
 	    }
 	}
 	RTWPData -
@@ -988,36 +1184,40 @@ proc SentPacket {pid} {
 		    incr PkOutCount -1
 		    set PkOutState [lindex $PkOutSaved 1]
 		    set PkOutData [lindex $PkOutSaved 2]
-		    SendPacket RTHeader [PrepData]
+		    foreach "ts vs" [PrepData] {}
+		    $ProcSendPacket RTHeader $ts $vs
 		} else {
-		    SendPacket XfrCmpl [list int $CMD($PkOutStart)]
+		    $ProcSendPacket XfrCmpl int $CMD($PkOutStart)
 		}
 	    } else {
 		incr PkOutCount -1
 		if { $CurrPSPID(RT) != "A201" } {
-		    SendPacket RTWPData [PrepData]
+		    set p RTWPData
 		} elseif { $pid != "RTLinkData" } {
 		    set PkOutState $CurrPSPID(RTLinkData)
-		    SendPacket RTLinkData [PrepData]
+		    set p RTLinkData
 		} else {
 		    set PkOutState $CurrPSPID(RTWPData)
-		    SendPacket RTWPData [PrepData]
+		    set p RTWPData
 		}
+		foreach "ts vs" [PrepData] {}
+		$ProcSendPacket $p $ts $vs
 	    }
 	}
 	TRHeader {
 	    if { $PkOutCount == 0 } {
 		Log "StP> count 0 after TRHeader"
-		SendPacket XfrCmpl [list int $CMD($PkOutStart)]
+		$ProcSendPacket XfrCmpl int $CMD($PkOutStart)
 	    } else {
 		incr PkOutCount -1
 		set PkOutState $CurrPSPID(TRData)
-		SendPacket TRData [PrepData]
+		foreach "ts vs" [PrepData] {}
+		$ProcSendPacket TRData $ts $vs
 	    }
 	}
 	TRData {
 	    if { $PkOutCount == 0 } {
-		SendPacket XfrCmpl [list int $CMD($PkOutStart)]
+		$ProcSendPacket XfrCmpl int $CMD($PkOutStart)
 	    } else {
 		incr PkOutCount -1
 		if { $PkOutSaved == "" } {
@@ -1028,7 +1228,8 @@ proc SentPacket {pid} {
 			}
 		    }
 		}
-		SendPacket $pid [PrepData]
+		foreach "ts vs" [PrepData] {}
+		$ProcSendPacket $pid $ts $vs
 	    }
 	}
 	XfrCmpl {
@@ -1040,10 +1241,10 @@ proc SentPacket {pid} {
 
 proc PrepData {} {
     # prepare WPs, RTs, or TRs data to be transferred to receiver
-    global PkOutState PkOutData PkOutSaved PkTRSegs PkSegData PkTRTPn PDTYPE \
-	    WPName WPCommt WPPosn WPDatum WPDate WPSymbol WPDispOpt WPAlt \
-	    WPHidden RTIdNumber RTCommt TRName TRDatum TRTPoints TRSegStarts \
-	    TRColour TRHidden DataIndex
+    global PkOutState PkOutData PkOutInfo PkOutSaved PkTRSegs PkSegData \
+	    PkTRTPn PDTYPE WPName WPCommt WPPosn WPDatum WPDate WPSymbol \
+	    WPDispOpt WPAlt WPHidden RTIdNumber RTCommt TRName TRDatum \
+	    TRTPoints TRSegStarts TRColour TRHidden DataIndex
     #DJG contribution: Autonumber globals
     global RTSendNumber RTsSent
     #--
@@ -1059,10 +1260,10 @@ proc PrepData {} {
 			   $WPDatum($ix) "WGS 84"]
 	    }
 	    return [list $PDTYPE($PkOutState) \
-		             [PrepWPData $PkOutState \
-			          $WPName($ix) $WPCommt($ix) $p \
-				  $WPDate($ix) $WPSymbol($ix) \
-				  $WPDispOpt($ix) $WPAlt($ix) $WPHidden($ix)]]
+			[PrepWPData $PkOutState \
+			     $WPName($ix) $WPCommt($ix) $p \
+			     $WPDate($ix) $WPSymbol($ix) \
+			     $WPDispOpt($ix) $WPAlt($ix) $WPHidden($ix)]]
 	}
 	D200 -  D201 -  D202 {
 	    # RT header
@@ -1355,6 +1556,29 @@ proc UnPackData {data types} {
 		    set x [string trim $x " "]
 		}
 	    }
+	    string<* {
+		# null-terminated string of at most *-1 characters
+		# to be truncated if it exceeds the limit
+		regsub string< $t "" n
+		incr n -1
+		set x ""
+		for { set i 0 } { $i < $n } { incr i } {
+		    set c [lindex $data $i]
+		    binary scan $c c d
+		    if { $d == 0 } { break }
+		    append x $c
+		}
+		if { $d != 0 } {
+		    # find the actual length
+		    while 1 {
+			set c [lindex $data $i]
+			binary scan $c c d
+			if { $d == 0 } { break }
+			incr i
+		    }
+		}
+		set n [expr $i+1]
+	    }
 	    charray=* {
 		regsub charray= $t "" n
 		set x ""
@@ -1440,11 +1664,11 @@ proc UnPackData {data types} {
     return $vals
 }
 
-proc PackData {tvals} {
-    # convert from a pair with a list of types and a list of values into a
+proc PackData {types vals} {
+    # convert from a list of types and a list of values into a
     #  list of bytes (as TCL characters)
 
-    return [split [DataToStr [lindex $tvals 0] [lindex $tvals 1]] ""]
+    return [split [DataToStr $types $vals] ""]
 }
 
 proc DataToStr {types vals} {
@@ -1503,6 +1727,16 @@ proc DataToStr {types vals} {
 	    string {
 		append data [binary format "a*x" $v]
 	    }
+	    string<* {
+		# null-terminated string of at most *-1 characters
+		# to be truncated if it exceeds the limit
+		regsub string< $t "" n
+		incr n -1
+		if { [string length $v] > $n } {
+		    set v [string range $v 0 $n]
+		}
+		append data [binary format "a*x" $v]
+	    }
 	    charray=* {
 		regsub charray= $t "" n
 		append data [binary format "A$n" $v]
@@ -1549,8 +1783,9 @@ proc DataToStr {types vals} {
 		append data [DataToStr $t $v]
 	    }
 	    ignored {
+		# no need for this:
 		# just to ensure a non-empty packet data
-		append data [binary format "c" 0]
+		#append data [binary format "c" 0]
 	    }
 	    default {
 		Log "DataToStr> unimplemented data type when packing: $t"
@@ -1593,10 +1828,11 @@ proc FromGarminColour {pid code} {
 
 proc EndInProt {pid data} {
     # deal with end of input protocol
-    #  $pid in {A000 A001 A010 A100 A200-201 A300-302 A500 A600 A650 A700 A906}
+    #  $pid in {A000 A001 A010 A100 A200-201 A300-302 A500 A600 A650
+    #           A700 A906}
     #    (cf. ProcInPacket) but {A010 A500 A600 A650 A700} not supported here
-    # global $Request==get$wh  where $wh in {WP RT TR PosnData DtTmData}
-    #                          (cf. SendData)
+    # global $Request==get$wh  where $wh in {WP RT TR LAP PosnData
+    #                          DtTmData} (cf. SendData)
     #                         GPSOpResult is set to 0 on success or 1 on error
     #                ==check=$args  where 1st arg should be executed as
     #                          connection is ok
@@ -1671,14 +1907,15 @@ proc InDataWPRT {data pid} {
 
     set wps ""
     foreach d [ConvWPData $data $CurrPSPID($pid)] {
-	set name [lindex $d 0] ; set ix [IndexNamed WP $name]
+	set name [lindex $d 0]
+	if { ! [CheckName Ignore $name] } {
+	    if { [set nname [AskForName $name]] == "" } { continue }
+	    set d [AddOldNameToObs $d $name]
+	    set d [lreplace $d 0 0 $nname]
+	    set name $nname
+	}
+	set ix [IndexNamed WP $name]
 	if { $GetSet(WP) == "" || [lsearch -exact $GetSet(WP) $ix] != -1 } {
-	    if { ! [CheckName Ignore $name] } {
-		if { [set nname [AskForName $name]] == "" } { continue }
-		set d [AddOldNameToObs $d $name]
-		set d [lreplace $d 0 0 $nname]
-		set name $nname
-	    }
 	    set name [StoreWP $ix $name $d $GetDispl]
 	    lappend wps $name
 	}
@@ -1733,7 +1970,9 @@ proc ConvWPData {data pid} {
 		Alt {
 		    switch $pid {
 			D108 -  D109 -  D110 {
-			    if { $v > 1e10 } { set v "" }
+			    if { $v < 1e20 } {
+				set v [format %.1f $v]
+			    } else { set v "" }
 			}
 			D150 -  D151 -  D152 -  D154 -  D155 {
 			    # this depends on what HiddenGet does!
@@ -1805,7 +2044,8 @@ proc ConvRTHeaderData {data pid} {
 
     switch $pid {
 	D200 -  D202 {
-	    return [list $data ""]
+	    lappend data ""
+	    return $data
 	}
 	D201 {
 	    return $data
@@ -2054,6 +2294,34 @@ proc ConvLAPData {data pid} {
     return $r
 }
 
+proc InDataAL {data} {
+    # show almanac data to user or, if in command-line mode, store it as
+    #  text in global ALDataTxt
+    #  $data is a list of lists of values
+    global CurrPSPID DATAFOR TXT CMDLINE ALDataTxt
+
+    set pid $CurrPSPID(ALData)
+    set title "$TXT(nameAL)\n" ; set sep ""
+    foreach n $DATAFOR($pid,ns) {
+	append title $sep $TXT(alm_$n)
+	set sep ", "
+    }
+    if { $CMDLINE } {
+	set ALDataTxt "$title\n"
+    } else { DisplayInfo $title }
+    foreach lst $data {
+	set line "" ; set sep ""
+	foreach v $lst {
+	    append line $sep $v
+	    set sep " "
+	}
+	if { $CMDLINE } {
+	    append ALDataTxt $line "\n"
+	} else { DisplayInfo $line }
+    }
+    return
+}
+
 proc ConvGarminDate {gd} {
     # converts Garmin date (seconds since 1990.01.01 00:00:00) into list
     #  with date in current format and seconds since beginning of $YEAR0,
@@ -2452,7 +2720,7 @@ proc HiddenCode {type val} {
     #      "|R" preceded by shift sequence if any and followed by count as
     #       character with code in (32+1..94), and code proper
 
-    set chars [PackData [list $type [list $val]]]
+    set chars [PackData $type [list $val]]
     set r ""
     set shift 0
     while { $chars != "" } {
@@ -2770,9 +3038,11 @@ proc InitProtocols {prodid} {
     switch -glob $PRTCLDEF($prodid) {
 	array {
 	    foreach p $PSPROTOCOLS {
-		set CurrPSPID($p) [set pid $PSPID($prodid,$p)]
-		foreach dp $PROTREQ($pid) {
-		    set CurrPSPID($dp) $PSPID($prodid,$dp)
+		if { ! [catch {set pid $PSPID($prodid,$p)}] } {
+		    set CurrPSPID($p) $pid
+		    foreach dp $PROTREQ($pid) {
+			set CurrPSPID($dp) $PSPID($prodid,$dp)
+		    }
 		}
 	    }
 	    foreach d $PSPDEF($CurrPSPID(Link)) {
@@ -2800,7 +3070,7 @@ proc InitProtocols {prodid} {
 
 proc RecAdjustToProtocols {} {
     # adjust receiver parameters according to its protocols
-    global CurrPSPID RECTYPES PRTNUMID PRTHNID PTRNUMID PTRHNID
+    global CurrPSPID RECTYPES RECINFO PRTNUMID PRTHNID PTRNUMID PTRHNID
 
     # $RECTYPES does not contain LAP if $SUPORTLAPS is not set
     foreach t $RECTYPES {
@@ -2828,6 +3098,10 @@ proc RecAdjustToProtocols {} {
 	    }
 	}
     }
+    # is there is a protocol for AL data?
+    if { [catch {set $CurrPSPID(ALData)}] && $p == "N/A" } {
+	set RECINFO [Delete $RECINFO AL]
+    }
     return
 }
 
@@ -2964,10 +3238,10 @@ proc ResetAutoNumberRT {} {
 
 proc GPSChangeProtocol {prot} {
     # change current protocol
-    #  $prot in {garmin, nmea, stext, simul} (for Garmin)
+    #  $prot in {garmin, garmin_usb, nmea, stext, simul}
     # must change GPSProtocolExt if successful
-    global GPSProtocol GPSProtocolExt GPSState TXT NoGarmin SRLFILE Eof \
-	    RealTimeLogOn
+    global GPSProtocol GPSProtocolExt GPSState NoGarmin RealTimeLogOn TXT \
+	GPSProtocolButton
 
     if { $GPSProtocol == $prot } { return }
     if { ! $NoGarmin } {
@@ -2976,23 +3250,21 @@ proc GPSChangeProtocol {prot} {
 	set NoGarmin 1
     }
     if { $GPSState == "online" } {
-	set Eof 1
-	set GPSState offline
-	close $SRLFILE
-	DisableGPS
+	CloseConnection
     }
     if { $RealTimeLogOn } {
 	GPSRealTimeLogOnOff
     }
     set GPSProtocol $prot ; set GPSProtocolExt $TXT($prot)
+    set GPSProtocolButton $prot
     return
 }
 
 proc AbortComm {args} {
     # abort communication in progress
     #  $args either void or a message id to be shown
-    # global $Request==get$wh  where $wh in {WP RT TR PosnData DtTMData}
-    #                          (cf. SendData)
+    # global $Request==get$wh  where $wh in {WP RT TR LAP PosnData
+    #                          DtTmData} (cf. SendData)
     #                         GPSOpResult is set to 1 (error)
     #                ==check=$args  where 2nd arg should be executed as
     #                         connection is apparently down;
