@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: garmin_protocols.tcl
-#  Last change:  22 November 2004
+#  Last change:  10 April 2006
 #
 
 # description of Garmin protocols based on Garmin GPS Interface
@@ -25,6 +25,11 @@
 # updated for:
 #   Garmin GPS Interface Specification Version 1 Rev. 3
 #   Garmin GPS Interface Specification Version 1 Rev. A
+#   Garmin GPS Interface Specification Version 1 Rev. B
+
+# Garmin USB protocol is implemented assuming that the USB port is driven by
+#  the garmin_gps Linux kernel driver by Hermann Kneissel and part of the
+#  official 2.6 Linux kernel releases
 
    # ASCII codes (decimal)
 set DLE 16
@@ -40,11 +45,16 @@ array set PROTTAG {
     P Physical  L Link  A Application  D Data  T nospec
 }
 
+   # categories of known product specific protocols
+set PSPROTOCOLS {Link DevCmd WP WPCat RT TR LAP Prx AL DtTm
+                 FlightBook Posn PVT}
+
    # application protocol categories
 array set PROTCAT {
     A010  DevCmd
     A011  DevCmd
     A100  WP
+    A101  WPCat
     A200  RT
     A201  RT
     A300  TR
@@ -63,7 +73,10 @@ array set PROTCAT {
     A902  nospec
     A903  nospec
     A904  nospec
+    A905  nospec
     A906  LAP
+    A907  nospec
+    A914  nospec
 }
 
    # protocol requirements
@@ -76,6 +89,7 @@ array set PROTREQ {
     A010  ""
     A011  ""
     A100  WPData
+    A101  WPCatData
     A200  {RTHeader RTWPData}
     A201  {RTHeader RTWPData RTLinkData}
     A300  TRData
@@ -94,7 +108,27 @@ array set PROTREQ {
     A902  ""
     A903  ""
     A904  ""
+    A905  D?
     A906  LAPData
+    A907  {D? D? D? D?}
+    A914  ""
+}
+
+   # Garmin USB Protocol
+array set GUSB {
+    LAYERID,TRANSPORT  0
+    LAYERID,APPL 20
+    LAYERID,PRIVATE  0x01106E4B
+    PRIV,PID,SET,DEBUG    1
+    PRIV,PID,SET,MODE     2
+    PRIV,PID,INFO,REQ     3
+    PRIV,PID,INFO,RESP    4
+    PRIV,PID,RESET,REQ    5
+    PRIV,PID,SET,DEF,MODE 6
+    MODE,NATIVE          0
+    MODE,GARMIN,SERIAL   1
+    PID,TRANSPORT,START,SESSION,REQ 5
+    PID,TRANSPORT,START,SESSION,RESP 6
 }
 
    # Basic Link Protocol
@@ -103,24 +137,25 @@ set PID(NAK) 21 ; set RPID(21) NAK
 set PID(PArray) 253 ; set RPID(253) PArray
 set PID(PrdReq) 254 ; set RPID(254) PrdReq
 set PID(PrdData) 255 ; set RPID(255) PrdData
-
-   # names of known product specific protocols
-set PSPROTOCOLS {Link DevCmd WP RT TR Prx AL DtTm Posn PVT}
+set PID(PrdExtData) 248 ; set RPID(248) PrdExtData
 
    # definitions of Link product specific protocols
 set PSPDEF(L001) {{CmdData 10} {XfrCmpl 12} {DtTmData 14} {PosnData 17}
                   {PrxWPData 19} {Records 27} {RTHeader 29} {RTWPData 30}
                   {ALData 31} {TRData 34} {WPData 35} {PVTData 51}
                   {RTLinkData 98} {TRHeader 99} {FlightBook_Record 134}
-                  {LAPData 149}}
+                  {LAPData 149} {WPCatData 152}}
 set PSPDEF(L002) {{ALData 4} {CmdData 11} {XfrCmpl 12} {DtTmData 20}
                   {PosnData 24} {PrxWPData 27} {Records 35} {RTHeader 37}
                   {RTWPData 39} {WPData 43}}
 
+    # packet ids not described in the specifications:
+    #   114: appears before a PVTData packet in the Quest
+
     # definition of product specific device commands
 set PSCMDDEF(A010) {{Abort 0} {XfrAL 1} {XfrPosnData 2} {XfrPrx 3} {XfrRT 4}
                   {XfrDtTmData 5} {XfrTR 6} {XfrWP 7} {TurnOff 8} {StartPVT 49}
-                  {StopPVT 50} {XfrFlightBook 92} {XfrLAP 117}}
+                  {StopPVT 50} {XfrFlightBook 92} {XfrLAP 117} {XfrWPCat 121}}
 set PSCMDDEF(A011) {{Abort 0} {XfrAL 1} {XfrRT 8} {XfrPrx 17} {XfrDtTmData 20}
                   {XfrWP 21} {TurnOff 26}}
 
@@ -149,7 +184,8 @@ set PDTYPE(NAK) union=int,byte
 array set PDTYPE {
     PArray  starray=char,word
     PrdReq  ignored
-    PrdData  {int int string ignored}
+    PrdData    {int int string ignored}
+    PrdExtData ignored
     Records  int
     D100  {charray=6 semicircle unused=4 charray=40}
     D101  {charray=6 semicircle unused=4 charray=40 float byte}
@@ -167,6 +203,7 @@ array set PDTYPE {
     D110  {byte byte byte byte int bytes=18 semicircle float float float
            charray=2 charray=2 longword float longword int string string string
 	   string string string}
+    D120  {string<17}
     D150  {charray=6 charray=2 byte semicircle int charray=24 charray=2
 	   charray=30 charray=40}
     D151  {charray=6 semicircle unused=4 charray=40 float charray=30
@@ -280,27 +317,35 @@ array set PRODID {
     "GPSMAP 295" 128
     "GPS 12 Map" 129
     eTrex  130
+    "GPSMAP 176" 136
     "GPS 12 4.58" 138
-    "eTrex Summit"  141
+    "eTrex Summit"  {141 295}
     "GPSMAP 196" 145
     "StreetPilot 3" 151
     "eTrex Venture/Mariner"  154
     "GPS 5"  155
     "eTrex Euro"  156
-    "eTrex Vista" 169
+    "eTrex Vista" {169 420}
     "GPS 76" 173
     "GPSMAP 76" 177
-    "eTrex Legend" 179
+    "eTrex Legend" {179 411}
     "GPSMAP 76S" 194
     "Rino 110"   197
     "Rino 120"   209
+    "Quest" 231
     "GPS 72" 247
     "Geko 201" 248
     "Geko 301" 256
+    "Rino 130" 264
     "Forerunner" 282
+    "Forerunner 301" 283
     "GPSmap 276C" 285
+    "GPS 60" 289
     "GPSMap60C" 291
     "ForeTrex" 314
+    "eTrex Vista C" 315
+    "eTrex Legend C" 315
+    "eTrex Venture" 419
 }
 
     # product names and how to obain its product specific protocols
@@ -370,6 +415,7 @@ array set PNAME {
     128  "GPSMAP 295"
     129  "GPS 12 Map"
     130  eTrex
+    136  "GPSMAP 176"
     138  "GPS 12"
     141  "eTrex Summit"
     145  "GPSMAP 196"
@@ -384,13 +430,22 @@ array set PNAME {
     194  "GPSMAP 76S"
     197  "Rino 110"
     209  "Rino 120"
+    231  "Quest"
     247  "GPS 72"
     248  "Geko 201"
     256  "Geko 301"
+    264  "Rino 130"
     282  "Forerunner"
+    282  "Forerunner 301"
     285  "GPSmap 276C"
+    289  "GPS 60"
     291  "GPSMap60C"
+    295  "eTrex Summit"
     314  "ForeTrex"
+    315  "eTrex Vista/Legend C"
+    411  "eTrex Legend"
+    419  "eTrex Venture"
+    420  "eTrex Vista"
 }
 
 # the descriptions below are valid for all versions except:
@@ -400,8 +455,10 @@ array set PNAME {
 #  36+ version >= 3.0
 #  77, 77+, 77++ versions < 3.01, < 3.50, < 3.61
 #  77+++  version >= 3.61
-#  111 version <= 2.7
-#  111+ version > 2.7
+#  111 version < 2.7
+#  111+ version >= 2.7
+#  285 version < 3.0 (just a guess)
+#  285+ version >= 3.0
 
       # the pairs in the following entries have a comparison to be made
       #  to the receiver version and the identifier to be used if the
@@ -409,8 +466,9 @@ array set PNAME {
 array set PRTCLVERSION {
     29  {{{>= 400} 29+}}
     36  {{{>= 300} 36+}}
-    77  {{{>=361} 77+++} {{>= 350} 77++} {{>= 301} 77+}}
+    77  {{{>= 361} 77+++} {{>= 350} 77++} {{>= 301} 77+}}
     111 {{{>= 270} 111+}}
+    285 {{{>= 300} 285+}}
 }
 
 array set PRTCLDEF {
@@ -471,6 +529,7 @@ array set PRTCLDEF {
     128  see=111
     129  array
     130  diff
+    136  diff
     138  see=77+
     141  see=130
     145  array
@@ -485,13 +544,23 @@ array set PRTCLDEF {
     194  see=177
     197  diff
     209  see=197
+    231  array
     247  see=116
     248  array
     256  diff
+    264  diff
     282  array
+    283  array
     285  see=291
+    285+ array
+    289  see=315
     291  array
+    295  see=248
     314  array
+    315  array
+    411  diff
+    419  array
+    420  see=411
 }
 
 ### there is no information on which receivers use the PVT protocol
@@ -666,6 +735,30 @@ array set PSPID {
     177,PVTData D800
     177,nospec A900+A902+A903
 
+    231,Link L001
+    231,DevCmd A010
+    231,WP A100
+    231,WPData D110
+    231,WPCat A101
+    231,WPCatData D120
+    231,RT A201
+    231,RTHeader D202
+    231,RTWPData D110
+    231,RTLinkData D210
+    231,TR A301
+    231,TRHeader D312
+    231,TRData D302
+    231,Prx N/A
+    231,AL A500
+    231,ALData D501
+    231,DtTm A600
+    231,DtTmData D600
+    231,Posn A700
+    231,PosnData D700
+    231,PVT A800
+    231,PVTData D800
+    231,nospec T+A601+D601+A900+A902+A903+A904+A905
+
     248,Link L001
     248,DevCmd A010
     248,WP A100
@@ -673,7 +766,7 @@ array set PSPID {
     248,RT A201
     248,RTHeader D202
     248,RTWPData D108
-    248.RTLinkData D210
+    248,RTLinkData D210
     248,TR A301
     248,TRHeader D310
     248,TRData D301
@@ -708,6 +801,56 @@ array set PSPID {
     282,LAPData D906
     282,nospec A802+A903
 
+    283,Link L001
+    283,DevCmd A010
+    283,WP A100
+    283,WPData D110
+    283,RT A201
+    283,RTHeader D202
+    283,RTWPData D110
+    283,RTLinkData D210
+    283,TR A302
+    283,TRHeader D311
+    283,TRData D301
+    283,Prx  N/A
+    283,AL A500
+    283,ALData D501
+    283,DtTm A600
+    283,DtTmData D600
+    283,Posn A700
+    283,PosnData D700
+    283,PVT A800
+    283,PVTData D800
+    283,LAP A906
+    283,LAPData D906
+
+    285+,Link L001
+    285+,DevCmd A010
+    285+,WP A100
+    285+,WPData D110
+    285+,WPCat A101
+    285+,WPCatData D120
+    285+,RT A201
+    285+,RTHeader D202
+    285+,RTWPData D110
+    285+,RTLinkData D210
+    285+,TR A301
+    285+,TRHeader D312
+    285+,TRData D302
+    285+,Prx A400
+    285+,PrxWPData D110
+    285+,AL A500
+    285+,ALData D501
+    285+,DtTm A600
+    285+,DtTmData D600
+    285+,Posn A700
+    285+,PosnData D700
+    285+,PVT A800
+    285+,PVTData D800
+    285+,LAP A906
+    285+,LAPData D906
+    285+,nospec T+A601+D601+A802+D802A900+A902+A903+A904+A907+D907+D908+D909+D910
+
     291,Link L001
     291,DevCmd A010
     291,WP A100
@@ -752,6 +895,50 @@ array set PSPID {
     314,PVT A800
     314,PVTData D800
     314,nospec A802+D802+A903
+
+    315,Link L001
+    315,DevCmd A010
+    315,WP A100
+    315,WPData D109
+    315,RT A201
+    315,RTHeader D202
+    315,RTWPData D109
+    315,RTLinkData D210
+    315,TR A301
+    315,TRHeader D310
+    315,TRData D301
+    315,Prx A400
+    315,PrxWPData D109
+    315,AL A500
+    315,ALData D501
+    315,DtTm A600
+    315,DtTmData D600
+    315,Posn A700
+    315,PosnData D700
+    315,PVT A800
+    315,PVTData D800
+    315,nospec A601+D601+A900+A902+A903+A904+A907+D907+D908+D909+D910+A914
+
+    419,Link L001
+    419,DevCmd A010
+    419,WP A100
+    419,WPData D108
+    419,RT A201
+    419,RTHeade D202
+    419,RTWPData D108
+    419,RTLinkData D210
+    419,TR A301
+    419,TRHeader D310
+    419,TRData D301
+    419,Prx N/A
+    419,AL A500
+    419,ALData D501
+    419,DtTm A600
+    419,DtTmData D600
+    419,Posn A700
+    419,PosnData D700
+    419,PVT A800
+    419,PVTData D800
 }
 
           # must not differ on the Link and DevCmd protocols
@@ -774,11 +961,14 @@ array set PSDIFF {
     77  {59 {Prx A400} {PrxData D400} {ALData D501}}
     77+ {73 {Prx A400} {PrxData D403}}
    130  {111 {RTHeader D202}}
+   136  {177 {nospec A802+D802+A900+A902+A903}}
    111+ {111 {WPData D109} {RTWPData D109} {PrxWPData D109}}
    112  {59 {WPData D152} {RTWPData D152} {ALData D501}}
    155  {145 {nospec A900+A902+A903+A904}}
    197  {177 {Prx N/A}}
    256  {248 {nospec A802}}
+   264  {291 {nospec T001+A802+D802+A900+A902+A903}}
+   411  {419 {nospec A802+D802+A900+A902+A903}}
 }
 
 ## descriptions, positions in Garmin structures of GPSMan data
@@ -843,6 +1033,22 @@ array set DATAFOR {
     D312,ns {Colour Name}
     D312,ps {0 2}
 
+    D500,ns {week datatime clockc1 clockc2 ecc sqrta mnanom argprgee
+             rightasc rtrightasc inclin}
+    D500,ps {0 1 2 3 4 5 6 7 8 9 10}
+
+    D501,ns {week datatime clockc1 clockc2 ecc sqrta mnanom argprgee
+             rightasc rtrightasc inclin health}
+    D501,ps {0 1 2 3 4 5 6 7 8 9 10 11}
+
+    D550,ns {svid week datatime clockc1 clockc2 ecc sqrta mnanom argprgee
+             rightasc rtrightasc inclin}
+    D550,ps {0 1 2 3 4 5 6 7 8 9 10 11}
+
+    D551,ns {svid week datatime clockc1 clockc2 ecc sqrta mnanom argprgee
+             rightasc rtrightasc inclin health}
+    D551,ps {0 1 2 3 4 5 6 7 8 9 10 11 12}
+
     D906,ns {Start Dur Dist BegPosn EndPosn Cals TRIx}
     D906,ps {0 1 2 3 4 5 6}
 }
@@ -939,7 +1145,7 @@ array set HIDDENFOR {
 	     longword float longword int string string string string}
     D110,vs {1 0 0 0x80
 	"0 0 0 0 0 0 255 255 255 255 255 255 255 255 255 255 255 255"
-	1.0e25 "" "" 0xffffffff 1.0e25 0xfffffffff 0 "" "" "" ""}
+	1.0e25 "" "" 0xffffffff 1.0e25 0xffffffff 0 "" "" "" ""}
     D110,ps {0 1 2 3 5 8 10 11 12 13 14 15 18 19 20 21}
     D110,cs {
 	{ if { $class==0 || $class>0x46 } { lappend undef city facility } }
