@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,9 +17,9 @@
 #      along with this program.
 #
 #  File: langnl.tcl
-#  Last change:  26 October 2004
+#  Last change:  26 April 2006
 #  Dutch language file by Rob Buitenhuis (rob@buitenhs.demon.nl)
-#  last input from him inserted: 22 October 2004
+#  last input from him inserted: 26 April 2006
 #
 # 
 #  file langengl.tcl is consulted first; no need for duplicating entries here
@@ -287,6 +287,9 @@ array set MESS {
     undefinedTR   "Onbekende TR"
     emptymenu "Menu leeg; doorgaan?"
     cantwrtsymenu "Kan menu met symbolen niet wegschrijven"
+    cantwrtmbaks "Kan het achtergrondenbestand niet schrijven"
+    abbrevhasspaces "Afgekorte naam mag geen spaties bevatten"
+    getAL   "Verkrijgen almanak"
 }
 
 set TXT(RTcompflds) "# WP {$DTUNIT} graad {$ALUNIT} et. beschr."
@@ -849,7 +852,6 @@ array set TXT {
     PRJAPOLY "Amerik. polyconische"
     computations Berekeningen
     comparea "Berekenen oppervlakte"
-    optTRNUMBERINTVL "Toon TP nummer op de kaart elke"
     centre  Centrum
     mkclusters "Maak clusters"
     dlat "Waardenbereik v.d. breedte"
@@ -1191,6 +1193,31 @@ array set TXT {
     calrs "Calorieen"
     getLAP  "Ophalen rondes"
     lapsrun "Wedstrijd"
+    fromfile "van file"
+    fromdef "van definitie"
+    mbaktoload "Te laden achtergrond"
+    none "geen"
+    opt_Fonts "Font-groottes"
+    optFONTSIZE "Default font-grootte"
+    optFIXEDFONTSIZE "Vaste font-grootte"
+    optICONSIZE "Icon-grootte"
+    optTRNUMBERINTVL "Toon TP-info op kaart bij elk"
+    optTRINFO "Te tonen TP-info"
+    nameInfo "Informatie"
+    nameAL "Almanak"
+    alm_svid "sateliet-id"
+    alm_week "week"
+    alm_datatime "geg ref tijd"
+    alm_clockc1 "klok corr coeff s"
+    alm_clockc2 "klok corr coeff s/s"
+    alm_ecc "excentriciteit"
+    alm_sqrta "vwortel a"
+    alm_mnanom "gem onregelmatigheid"
+    alm_argprgee "arg perigeum"
+    alm_rightasc "rechte klimming"
+    alm_rtrightasc "snelh r klimming"
+    alm_inclin "inclinatie"
+    alm_health "geldigheid"
 }
 
     # the following definitions must be coherent with $TXT
