@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: langport.tcl
-#  Last change:  22 October 2004
+#  Last change:  6 April 2006
 #
 
 # file langengl.tcl is consulted first; no need for duplicating entries here
@@ -128,6 +128,7 @@ array set MESS {
     getRT   "A buscar Rotas"
     getTR   "A buscar Trajectos"
     getLAP  "A buscar Voltas"
+    getAL   "A buscar almanaque"
     putWP   "A pr Pontos"
     putRT   "A pr Rotas"
     putTR   "A pr Trajectos"
@@ -285,6 +286,8 @@ array set MESS {
     undefinedTR "TR no existente"
     emptymenu "Menu vazio; continuar?"
     cantwrtsymenu "No posso escrever no ficheiro do menu de smbolos"
+    cantwrtmbaks "No posso escrever no ficheiro de mapas"
+    abbrevhasspaces "Abreviatura no pode ter espaos"
 }
 
 set TXT(RTcompflds) "# PT {$DTUNIT} grau {$ALUNIT} etapa etiq"
@@ -330,6 +333,7 @@ array set TXT {
     nameMap   Mapa
     nameRTComp   "Clculo para Rota"
     nameTRComp   "Clculo para Trajecto"
+    nameInfo Informao
     GPSrec   "Receptor GPS"
     turnoff   Desligar
     get   Buscar
@@ -430,7 +434,8 @@ array set TXT {
     optACCFORMULAE "Frmulas de maior preciso"
     optASKPROJPARAMS "Rever parmetros da projeco"
     optBalloonHelp "Bales de ajuda"
-    optTRNUMBERINTVL "Mostrar nmero de PT de TR cada"
+    optTRNUMBERINTVL "Mostrar informao de PT de TR cada"
+    optTRINFO "Informao de PT a mostrar"
     opt_Formats   "Unidades e formatos"
     optDISTUNIT   "Distncia"
     optUSESLOWOPWINDOW "Janela de controle de operaes lentas"
@@ -486,6 +491,9 @@ array set TXT {
     optMAPCOLOUR,anim "Animao no mapa"
     optMAPCOLOUR,emptygrid  "Sem imagem"
     optMAPCOLOUR,fullgrid   "Com imagem"
+    opt_Fonts "Tamanhos de letra"
+    optFONTSIZE "Tamanho de letra, habitual"
+    optFIXEDFONTSIZE "Tamanho de letra, fixa"
     optMAPFONTSIZE "Tamanho de letra, mapa"
     optDEFTRTWIDTH "Espessura RT"
     optDEFTTRWIDTH "Espessura TR"
@@ -498,6 +506,7 @@ array set TXT {
     optPAPERSIZE "Tamanho do papel"
     optPRINTCMD "Comando para imprimir"
     optMapGuideVersion "Verso do MapGuide"
+    optICONSIZE "Tamanho dos smbolos"
     red   Verm
     green   Verde
     blue   Azul
@@ -533,9 +542,8 @@ array set TXT {
     TRlinedispl   "Desenhar resultado"
     TRTRdispl   "Desenhar TR"
     WP   PT
-    RT   RT
-    TR   TR
-    GR   GR
+    TP   PT
+    LP   PL
     LAP  VLT
     commrec   "Comunicao com receptor"
     abort   Desistir
@@ -1204,6 +1212,24 @@ array set TXT {
     clssbmn  "Fechar sub-menu"
     syhlpbx "Usar boto direito para\nreordenar elementos da lista"
     lapsrun Corrida
+    fromfile "de ficheiro"
+    fromdef "de definio"
+    mbaktoload "Carregar mapa"
+    none "nenhum"
+    nameAL "Almanaque"
+    alm_svid "id satlite"
+    alm_week "semana"
+    alm_datatime "data ref dados"
+    alm_clockc1 "coef corr relgio s"
+    alm_clockc2 "coef corr relgio s/s"
+    alm_ecc "excentricidade"
+    alm_sqrta "raz quadr a"
+    alm_mnanom "anomalia mdia"
+    alm_argprgee "arg perigeu"
+    alm_rightasc "asceno recta"
+    alm_rtrightasc "taxa de asc recta"
+    alm_inclin "inclinao"
+    alm_health "sade"
 }
 
     # the following definitions must be coherent with $TXT
