@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: main.tcl
-#  Last change:  7 December 2004
+#  Last change: 8 May 2006
 #
 # Includes contributions by
 #  - Brian Baulch (baulchb@onthenet.com.au) marked "BSB contribution"
@@ -26,9 +26,9 @@
 #  - David Gardner (djgardner@users.sourceforge.net) marked "DJG contribution"
 #
 
-set VERSIONDATE(day) 7
-set VERSIONDATE(month) 12
-set VERSIONDATE(year) 2004
+set VERSIONDATE(day) 8
+set VERSIONDATE(month) 5
+set VERSIONDATE(year) 2006
 
       # these tests here prevent errors in the lang*.tcl files
 if { [lsearch -exact "KM NAUTMILE STATMILE" $DISTUNIT] == -1 } {
@@ -137,15 +137,31 @@ if { int($TimeOffset) != $TimeOffset } {
     exit 1
 }
 
-set VERSION 6.2.1
+set VERSION 6.3
 
-set UDatums "" ; set UEllipsoids ""
 set WindowStack ""
 
+# fonts needed for proc AboutInfo
+
+set FontSize [font actual default -size]
+
+if { $FONTSIZE != "Tcl/Tk" } {
+    set f [font actual default]
+    set f [eval font create $f]
+    font configure $f -size $FONTSIZE
+    option add *Font $f
+}
+if { $FIXEDFONTSIZE != "Tcl/Tk" } {
+    set FixedFont "fixed $FIXEDFONTSIZE"
+} else { set FixedFont fixed }	    
+
+##
+
 proc AboutInfo {w} {
     # create widgets for displaying information about GPSMan in window $w
     # all widgets will be inside frame $w.fr and are packed
-    global Logo VERSION VERSIONDATE DateFormat COLOUR EPOSX EPOSY TXT
+    global Logo VERSION VERSIONDATE DateFormat COLOUR EPOSX EPOSY TXT \
+	FixedFont
 
     frame $w.fr -relief flat -borderwidth 5 -bg $COLOUR(messbg)
     if { $Logo != "" } {
@@ -157,7 +173,7 @@ proc AboutInfo {w} {
     label $w.fr.title -text "$TXT(GMtit) $VERSION / $d" -relief groove
     set frc $w.fr.frc
     frame $frc -relief flat -borderwidth 0
-    text $frc.txt -width 80 -height 12 -font fixed -wrap word \
+    text $frc.txt -width 80 -height 12 -font $FixedFont -wrap word \
 	    -yscrollcommand "$frc.tscrl set"
     bind $frc.txt <space> "$frc.txt yview scroll 1 pages ; break"
     bind $frc.txt <Key-Delete> "$frc.txt yview scroll -1 pages ; break"
@@ -199,16 +215,10 @@ set SYUSRMENU ""
 if { [file readable [set UFile(symenu) [file join $USERDIR symenu]]] } {
     LoadSymbolCustomMenu
 }
-InitSymbolImages
-
-foreach v "Logo SmallLogo" f "gpsman-logo.gif gpsman-logo-small.gif" {
-    if { [catch {set $v [image create photo \
-	    -file [file join $SRCDIR gmicons $f]]}] } {
-	set $v ""
-    }
-}
+InitImages
 
 ### on Unix systems GPSMan cannot be executed as root for security reasons
+###  this can be disabled by replacing the condition in the if by  0  (zero)
 if { $UNIX && $env(USER) == "root" } {
     GMMessage $MESS(cantexecasroot)
     exit 1
@@ -278,6 +288,8 @@ set OBSHEIGHT 3
 
 set DATEWIDTH $DATEW($DateFormat)
 
+set DATUMWIDTH 32
+
 set SUBDTUNIT $DLUNIT($DISTUNIT,subdist)
 
   # crosshair cursor centre in pixels from lower right corner
@@ -320,9 +332,8 @@ proc GMInit {} {
 	    GetDispl MAPKNOWNPROJS RECPROTOCOLS SYMBOLIMAGE RcMenu \
 	    GPSProtocol GPSProtocolExt RealTimeLogAnim PVTState USvState \
 	    MAPFTSIZES GSHPVersion NoImgLib ImgOutFormats SmallLogo UNIX GMEd \
-	    FILEFORMAT MapScaleButton MapPFDatum AutoNumRts SUPPORTLAPS
-
-    GlobalOptionsBindings
+	    FILEFORMAT MapScaleButton MapPFDatum AutoNumRts SUPPORTLAPS \
+	    FixedFont DATUMWIDTH GPSProtocolButton RECINFO
 
     set MapBounds "0 0 $MAPWIDTH $MAPHEIGHT"
     set MapRange(x) $MAPWIDTH ; set MapRange(y) $MAPHEIGHT
@@ -387,7 +398,7 @@ proc GMInit {} {
 	    menu .lsw.frmb.dtitle.m \
 		 -postcommand "FillDatumMenu .lsw.frmb.dtitle.m ChangeMapDatum"
 	    label .lsw.frmb.datum -textvariable Datum \
-		    -width [Measure "Astro B4 Sorol Atoll"]
+		    -width $DATUMWIDTH
 
 	    set t [Measure $TXT(GPSrec)]
 	    frame .lsw.frmb.frb
@@ -506,7 +517,7 @@ proc GMInit {} {
     frame $MpW.frm.frmap3.fr3
     set fr3 $MpW.frm.frmap3.fr3
     canvas $fr3.cv -width $MAPSCLENGTH -height 15 -relief flat
-    label $fr3.cv.val -width 10 -font fixed
+    label $fr3.cv.val -width 10 -font $FixedFont
     $fr3.cv create line 0 7.5 $MAPSCLENGTH 7.5 -arrow both \
 	    -fill $MAPCOLOUR(mapsel)
     $fr3.cv create line 1 0 1 15 -fill $MAPCOLOUR(mapsel)
@@ -609,7 +620,7 @@ proc GMInit {} {
 		"FillPFormtMenu $MpW.frmi.frmbs.grid.m ChangeMapPFormat"
 	    set mb $MpW.frmi.frmbs.grdatum
 	    menubutton $mb -textvariable MapPFDatum \
-		    -width [Measure "Astro B4 Sorol Atoll"] -menu $mb.m
+		    -width $DATUMWIDTH -menu $mb.m
 	    menu $mb.m -postcommand "FillDatumMenu $mb.m ChangeMPFDatum"
 
 	    label $MpW.frmi.frmbs.zone -textvariable MapZone -width 4 \
@@ -639,14 +650,20 @@ proc GMInit {} {
 		    -state disabled -width $bl
 	    menubutton $MpW.frmi.frmbs.mback -text $TXT(backgrnd) -width $bl \
 		    -menu $MpW.frmi.frmbs.mback.m
-	    menu $MpW.frmi.frmbs.mback.m
+	    set mbmn $MpW.frmi.frmbs.mback.m
+	    menu $mbmn
 	    # order of entries used in file map.tcl
-	    foreach f [list $TXT(loadmback) $TXT(savemback) $TXT(chgmback) \
+	    $mbmn add cascade -label $TXT(loadmback) -menu $mbmn.m
+	    menu $mbmn.m
+	    $mbmn.m add command -label $TXT(fromfile) -command LoadMapBack
+	    menu $mbmn.m.defs \
+		-postcommand [list FillBackgrndMenu $mbmn.m.defs LoadMapBack]
+	    $mbmn.m add cascade -label $TXT(fromdef) -menu $mbmn.m.defs	    
+	    foreach f [list $TXT(savemback) $TXT(chgmback) \
 		            $TXT(clearmback)] \
-		    c "LoadMapBack SaveMapBack ChangeMapBack ClearMapBack" \
+		    c "SaveMapBack ChangeMapBack ClearMapBack" \
 		    s "normal disabled disabled disabled" {
-		$MpW.frmi.frmbs.mback.m add command -label $f \
-			-command $c -state $s
+		$mbmn add command -label $f -command $c -state $s
 	    }
 	    button $MpW.frmi.frmbs.mclear -text $TXT(clear) -state disabled \
 		    -width $bl \
@@ -665,7 +682,7 @@ proc GMInit {} {
 	    set dm $MpW.frmi.frcoords.dtitle.m
 	    menu $dm -postcommand "FillDatumMenu $dm ChangeMapDatum"
 	    label $MpW.frmi.frcoords.datum -textvariable Datum \
-		    -width [Measure "Astro B4 Sorol Atoll"]
+		    -width $DATUMWIDTH
 	    menubutton $MpW.frmi.frcoords.ptitle -text $TXT(projection) \
 		    -menu $MpW.frmi.frcoords.ptitle.m
 	    menu $MpW.frmi.frcoords.ptitle.m \
@@ -808,10 +825,20 @@ proc GMInit {} {
 		    -state disabled
 	    $fr0.map.m add cascade -label $TXT(backgrnd) -menu $fr0.map.mback
 	    menu $fr0.map.mback
-	    foreach f [list $TXT(loadmback) $TXT(savemback) $TXT(chgmback) \
+	    $fr0.map.mback add cascade -label $TXT(loadmback) \
+		-menu $fr0.map.mback.m
+	    menu $fr0.map.mback.m
+	    $fr0.map.mback.m add command -label $TXT(fromfile) \
+		-command LoadMapBack
+	    menu $fr0.map.mback.m.defs \
+		-postcommand [list FillBackgrndMenu $fr0.map.mback.m.defs \
+				  LoadMapBack]
+	    $fr0.map.mback.m add cascade -label $TXT(fromdef) \
+		-menu $fr0.map.mback.m.defs	    
+	    foreach f [list $TXT(savemback) $TXT(chgmback) \
 		            $TXT(clearmback)] \
-		    c "LoadMapBack SaveMapBack ChangeMapBack ClearMapBack" \
-		    s "normal disabled disabled disabled" {
+		    c "SaveMapBack ChangeMapBack ClearMapBack" \
+		    s "disabled disabled disabled" {
 		$fr0.map.mback add command -label $f -command $c -state $s
 	    }
 	    $fr0.map.m add command -label $TXT(clear) -command ClearMap \
@@ -834,7 +861,7 @@ proc GMInit {} {
 	    set fr4 $MpW.frm.frmap3.fr4
 
 	    menubutton $fr4.datum -textvariable Datum -menu $fr4.datum.m \
-		    -width [Measure "Astro B4 Sorol Atoll"]
+		    -width $DATUMWIDTH
 	    menu $fr4.datum.m \
 		    -postcommand "FillDatumMenu $fr4.datum.m ChangeMapDatum"
 	    menubutton $fr4.proj -textvariable MapProjTitle -width 20 \
@@ -856,7 +883,7 @@ proc GMInit {} {
 	    label $fr5.xcoord -textvariable XCoord -width 11 -relief sunken
 	    label $fr5.ycoord -textvariable YCoord -width 11 -relief sunken
 	    menubutton $fr5.grdat -textvariable MapPFDatum \
-		    -width [Measure "Astro B4 Sorol Atoll"] -menu $fr5.grdat.m
+		    -width $DATUMWIDTH -menu $fr5.grdat.m
 	    menu $fr5.grdat.m \
 		    -postcommand "FillDatumMenu $fr5.grdat.m ChangeMPFDatum"
 
@@ -887,16 +914,27 @@ proc GMInit {} {
     $mn add separator
     $mn add cascade -label Datum -menu $mn.dat
     menu $mn.dat
-    $mn.dat add command -label $TXT(define) -command DefineDatum
-    menu $mn.dat.mn \
-	    -postcommand "FillDatumMenu $mn.dat.mn OpenDatum"
+    $mn.dat add command -label $TXT(define) -command "Define datum"
+    menu $mn.dat.mn -postcommand [list FillDatumMenu $mn.dat.mn \
+				      [list Inspect datum]]
     $mn.dat add cascade -label "$TXT(open)..." -menu $mn.dat.mn
     $mn add cascade -label $TXT(ellpsd) -menu $mn.ell
     menu $mn.ell
-    $mn.ell add command -label $TXT(define) -command DefineEllipsoid
-    menu $mn.ell.mn \
-	    -postcommand "FillEllipsoidMenu $mn.ell.mn OpenEllipsoid"
+    $mn.ell add command -label $TXT(define) -command "Define ellpsd"
+    menu $mn.ell.mn -postcommand [list FillEllipsoidMenu $mn.ell.mn \
+				      [list Inspect ellpsd]]
     $mn.ell add cascade -label "$TXT(open)..." -menu $mn.ell.mn
+    # prepared for more of these...
+    foreach t "backgrnd" p "Backgrnd" {
+	$mn add separator
+	$mn add cascade -label $TXT($t) -menu $mn.$t
+	menu $mn.$t
+	$mn.$t add command -label $TXT(define) -command "Define $t"
+	menu $mn.$t.mn -postcommand [list Fill${p}Menu $mn.$t.mn \
+					 [list Inspect $t]]
+	$mn.$t add cascade -label "$TXT(open)..." -menu $mn.$t.mn
+    }
+    $mn add separator
     $mn add command -label $TXT(syusrmenu) -command SymbolCustomMenu
 
     if { $SmallLogo == "" } {
@@ -1034,10 +1072,11 @@ proc GMInit {} {
     foreach p $RECPROTOCOLS($MYGPS) {
 	$fprt.title.m add command -label $TXT($p) \
 		-command "GPSChangeProtocol $p"
-	$RcMenu.pm add radiobutton -label $TXT($p) -variable GPSProtocol \
-	    -value $p -selectcolor $COLOUR(check) \
+	$RcMenu.pm add radiobutton -label $TXT($p) -value $p \
+	    -variable GPSProtocolButton -selectcolor $COLOUR(check) \
 	    -command "GPSChangeProtocol $p"
     }
+    set GPSProtocolButton $GPSProtocol
     label $fprt.prot -textvariable GPSProtocolExt -width 16
     
     if { $MYGPS != "Magellan" } {
@@ -1095,7 +1134,7 @@ proc GMInit {} {
 	    $RcW.frgps.bs.off configure -state normal"
     $RcMenu add command -label $TXT(turnoff) -state disabled \
 	    -command TurnOff
-    set r [Measure $TXT(nameWP)]
+    set r [Measure $TXT(nameLAP)]
     frame $RcW.frget -borderwidth 2
     frame $RcW.frget.frget1
     # the following frame only children are the buttons for types in $RECTYPES
@@ -1152,6 +1191,17 @@ proc GMInit {} {
 	    }
 	}
     }
+    # other information types that can be got from the receiver
+    foreach i $RECINFO {
+	set n $TXT(name$i)
+	button $RcW.frget.frget2.get$i -text $n -width $r -state disabled \
+	    -command "GetFromGPS $i ; \
+	                 $RcW.frget.frget2.get$i configure -state normal"
+	$RcMenu.gm add command -label $n -command "GetFromGPS $i" \
+	    -state disabled
+	lappend gmlst rec,getmn,$i $gmix
+	incr gmix
+    }
     array set WConf $gmlst
     array set WConf $ptmlst
     set WConf(rec,getbs) [winfo children $RcW.frget.frget2]
@@ -1267,7 +1317,7 @@ proc GMInit {} {
 		-side top -pady 5
     }
     pack $RcW.frget.frget1.title $RcW.frget.frget1.displ -side left -padx 5
-    foreach i $RECTYPES {
+    foreach i [concat $RECTYPES $RECINFO] {
 	pack $RcW.frget.frget2.get$i -side left -pady 2
     }
     pack $RcW.frget.frget1 $RcW.frget.frget2 -side top
@@ -1353,10 +1403,10 @@ proc About {} {
 
 proc GMStart {} {
     # initialize (most) global variables
-    global Index Number Storage File Proc WorkDir FileTypes FDDatum \
+    global Index Number Storage File Proc FDDatum \
 	    TYPES PROCTYPES CursorsChanged MAPWIDTH MAPHEIGHT MAPW2 MAPH2 \
 	    AzimuthDegrees MapEmpty DataDefault DataIndex PositionFormat \
-	    DEFAULTSYMBOL DEFAULTDISPOPT Datum EMPTYSTR AnimNo \
+	    DEFAULTSYMBOL DEFAULTDISPOPT Datum EMPTYSTR Anim \
 	    BalloonX BalloonY USERDIR UProjs UFile RealTimeLogOn \
 	    RealTimeLogAnim PVTState USvState USvData USvMap \
 	    BITMAPS SERIALPORT DEFSPORT tcl_platform DEFTRTWIDTH DEFTTRWIDTH \
@@ -1364,22 +1414,32 @@ proc GMStart {} {
 	    MapFont MAPFONTSIZE Travelling Travel TRAVELFONTSIZE NAMEWIDTH \
 	    MapMeasure DEFMAPPROJ DEFMAPPFRMT ALTUNIT AltUnit GMEd PlotWNo \
 	    FILEFORMAT DEFTRECPROTOCOL MapPFDatum DEFMAPPFDATUM \
-	    RealTimeGettingFix
+	    RealTimeGettingFix UDatums UEllipsoids UBackgrnds
 
     # BSB contribution: globals needed for Lowrance receivers
     global UnusedICInx UnusedWPInx MAX MYGPS
 
-    # serial port
+    GlobalOptionsBindings
+
+    # I/O port
     if { $UNIX && $SERIALPORT == "" } {
 	set SERIALPORT $DEFSPORT
     }
 
     if { $MYGPS == "Garmin" } {
 	set GPSProtocol $DEFTRECPROTOCOL
+	if { [regsub {^usb=} $SERIALPORT "" p] } {
+	    # I/O port given as command argument
+	    set SERIALPORT $p
+	    set GPSProtocol garmin_usb
+	}
     } else { set GPSProtocol [lindex $RECPROTOCOLS($MYGPS) 0] }
     set GetDispl 0
 
-    # projections, datums and ellipsoids
+    # user definitions:
+    # projections, datums and ellipsoids, map backgrounds, WP categories
+    set UDatums "" ; set UEllipsoids "" ; set UBackgrnds ""
+
     if { [file readable [set UFile(datum,ell) [file join $USERDIR datums]]] } {
 	SetUpUserDatums
     }
@@ -1388,6 +1448,9 @@ proc GMStart {} {
     if { [file readable [set UFile(proj,grid) [file join $USERDIR projs]]] } {
 	SetupUserProjs
     }
+    if { [file readable [set UFile(backgrs) [file join $USERDIR mapbaks]]] } {
+	SetUpUserBackgrnds
+    }
 
     # data-base
     set Number(Data) 0
@@ -1399,25 +1462,28 @@ proc GMStart {} {
            #  altitude in user unit and user unit (as $ALTUNIT);
            #  empty list stands for undefined, last 2 elements may
            #  be missing if user unit is metre
+           # ??MBack is name of map background definition to load when item
+           #  is displayed and the map is empty; empty string means undefined;
+           #  not valid for GR and LAP
     set Storage(WP) {WPName WPNum WPCommt WPObs WPPFrmt WPPosn WPDatum
-	             WPDate WPSymbol WPDispOpt WPAlt WPHidden WPDispl}
+	             WPDate WPSymbol WPDispOpt WPAlt WPMBack WPHidden WPDispl}
     set DataDefault(WP) [list "" -1 "" "" $PositionFormat "" $Datum \
-	    "" $DEFAULTSYMBOL $DEFAULTDISPOPT "" "" 0]
+	    "" $DEFAULTSYMBOL $DEFAULTDISPOPT "" "" "" 0]
     set Storage(RT) {RTIdNumber RTCommt RTObs RTWPoints RTStages RTWidth
-	             RTColour RTDispl}
-    set DataDefault(RT) [list "" "" "" "" "" $DEFTRTWIDTH $MAPCOLOUR(RT) 0]
+	             RTColour RTMBack RTDispl}
+    set DataDefault(RT) [list "" "" "" "" "" $DEFTRTWIDTH $MAPCOLOUR(RT) "" 0]
     set stRS {RScommt RSlabel RShidden}
     set DataDefault(RS) [list "" "" ""]
     set Storage(TR) {TRName TRObs TRDatum TRTPoints TRSegStarts TRHidden
-                     TRWidth TRColour TRDispl}
+                     TRWidth TRColour TRMBack TRDispl}
     set DataDefault(TR) [list "" "" $Datum "" "" "" $DEFTTRWIDTH \
-			     $MAPCOLOUR(TR) 0]
+			     $MAPCOLOUR(TR) "" 0]
     set stTP {TPlatd TPlongd TPlatDMS TPlongDMS TPdate TPsecs TPalt TPdepth}
     set DataDefault(TP) [list "" "" "" "" "01-Jan-1988 00:00:00" 0 "" ""]
     set Storage(LN) {LNName LNObs LNDatum LNPFrmt LNLPoints LNSegStarts \
-	             LNWidth LNColour LNDispl}
+	             LNWidth LNColour LNMBack LNDispl}
     set DataDefault(LN) [list "" "" $Datum $PositionFormat "" "" $DEFTLNWIDTH \
- 	                      $MAPCOLOUR(LN) 0]
+ 	                      $MAPCOLOUR(LN) "" 0]
     set stLP {LPposn LPalt}
     set DataDefault(LP) [list "" ""]
     set Storage(GR) {GRName GRObs GRConts GRDispl}
@@ -1450,11 +1516,10 @@ proc GMStart {} {
     set GMEd(LN,ptname) LP
 
     # files
-    set FileTypes {WP RT TR LN LAP GR Data WPDistBear WPNearest RTComp TRComp
-                   Map Plot MapBkInfo Image PVTData}
-    foreach t $FileTypes { set File($t) "" }
+    set filetypes {WP RT TR LN LAP GR Data WPDistBear WPNearest RTComp TRComp
+                   Map Plot MapBkInfo Image PVTData Info}
+    foreach t $filetypes { set File($t) "" }
     set FDDatum ""
-    set WorkDir [pwd]
     set USvState [file join $USERDIR sstate]
     set USvData [file join $USERDIR sdata]
     set USvMap [file join $USERDIR smap]
@@ -1492,10 +1557,10 @@ proc GMStart {} {
     set RealTimeLogOn 0 ; set RealTimeLogAnim 0 ; set RealTimeGettingFix 0
     set PVTState abort
     set Travelling 0
-    set Travel(font) [FixedFont $TRAVELFONTSIZE]
+    set Travel(font) [GFixedFont $TRAVELFONTSIZE]
     array set Travel {
-	tosave {conf,1 conf,2 travel,cdsp font warn warn,* mindist chggparam
-	        chggwparam chggix}
+	tosave {conf,1 conf,2 travel,cdsp font warn warn,* mindist offroad
+	    chggparam chggwparam chggix}
 	conf,1  {lab+fix lab+speed c_trkcts c_trn c_vel_z lab+hour
 	lab+alt lab+nxtWP lab+prvWP lab+dist lab+vmg lab+ete lab+eta
 	lab+xtk lab+cts}
@@ -1525,6 +1590,7 @@ proc GMStart {} {
 	warn,xtk,max 5
 	warn,xtk,level low
 	mindist 0.1
+	offroad 0
 	chggparam 0.8
 	chggwparam 0.9
 	chggix 2
@@ -1586,11 +1652,11 @@ proc GMStart {} {
     # map
     set MapEmpty 1 ; set MapMeasure ""
     set MAPW2 [expr $MAPWIDTH/2.0] ; set MAPH2 [expr $MAPHEIGHT/2.0]
-    set AnimNo 0
+    set Anim(number) 0
     foreach x "high medium low" y "important normal info" {
 	set MAPCOLOUR(trvwrn$x) $MAPCOLOUR(trvwrn$y)
     }
-    set MapFont [FixedFont $MAPFONTSIZE]
+    set MapFont [GFixedFont $MAPFONTSIZE]
 
     MapProjectionIs $DEFMAPPROJ
     # order is important here
